@@ -468,7 +468,10 @@ as_chMDB <- function(x, tkcon, timestamp=Sys.time(), overwrite=FALSE, by=10^5){
    ## Rules of update ----
    tst <- list_chMDB_timestamps(tkcon, dbName)
    maxTs <- suppressWarnings(max(tst$timestamp))
-   if(!is.na(timestamp) && !is.null(maxTs) && timestamp - maxTs <= 60){
+   if(
+      !is.na(timestamp) && !is.null(maxTs) &&
+      difftime(timestamp, maxTs, units="secs") <= 60
+   ){
       stop(
          "Timestamp should be more recent (60 sec.) than those already recorded"
       )
