@@ -604,6 +604,7 @@ dims.fileMDB <- function(
          ncol=numeric(),
          nrow=numeric(),
          records=numeric(),
+         bytes=numeric(),
          transposed=logical()
       ))
    }
@@ -693,7 +694,7 @@ dims.fileMDB <- function(
                "table"
             ),
             ncol=n[2],
-            nrow=n[1],
+            nrow=n[1]
          ) %>% 
             dplyr::mutate(
                records=ifelse(
@@ -701,6 +702,7 @@ dims.fileMDB <- function(
                   as.numeric(.data$ncol) * as.numeric(.data$nrow),
                   .data$nrow
                ),
+               bytes=fs,
                transposed=FALSE
             )
       }
@@ -742,7 +744,7 @@ data_file_size <- function(x, hr=FALSE){
    df <- data_files(x)$dataFiles
    fs <- file.size(df)
    if(hr){
-      fs <- .format_file_size(fs)
+      fs <- .format_bytes(fs)
    }
    fc <- lapply(
       df, function(x){
@@ -1874,22 +1876,6 @@ DEFAULT_READ_PARAMS <- list(delim='\t', na="NA")
    return(d)
 }
 
-.format_file_size <- function(n){
-   sunits <- c("B", "KB", "MB", "GB", "TB")
-   nunits <- log2(n)%/%10
-   toRet <- lapply(
-      1:length(n),
-      function(i){
-         format(
-            n[i]/(2^(10*nunits[i])),
-            digit=1,
-            nsmall=ifelse(nunits[i]==0, 0, 1)
-         )
-      }
-   )
-   return(paste(toRet, sunits[nunits+1]))
-}
-
 .read_td <- function(
    tm,      # table model
    f,       # file name
